local TAG = "0.4.lua: "

-- При открытии исследования SCALAR

log(TAG .. "Level 0.4 script was successfully loaded")

local GateSide = luajava.bindClass(GNS .. "Gate$Side")

local sm -- systems.GraphicsSystem.storylineMessages
local ql -- systems.GraphicsSystem.questList
local setupListener
local disposeListener

local questListItem

local s6_sideMenuWasOffscreen

--

local tut
tut = tutorial:new({
    {-- Победить первую волну
        check = function()
            return systems.WaveSystem:getCompletedWavesCount() >= 1
        end,
        start = function()
            log("Stage 1 start")

            -- При вызове волны скрываем кнопку и останавливаем таймер волны
            tut:condition(function() return systems.WaveSystem.wave ~= nil end, function()
                systems.GraphicsSystem.mainUi:hideNextWaveButton()
                systems.WaveSystem:freezeTimeToNextWave(true)
            end)

            tut:delay(1, function()
                sm:queue({
                    tut.ADVINAS_TAG .. " In this tutorial, I'll show you [#00BCD4]teleports, barriers, resources and miners[].",
                    tut.ADVINAS_TAG .. " First, let's make some coins. I already put a tower for you, go on and [#00BCD4]defeat the first wave[] of Regular enemies."
                })
                sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                    run = function()
                        log("Stage 1 sm continue")

                        -- Показываем счетчик MDPS
                        systems.GraphicsSystem.mainUi:showMdps(true, nil)

                        -- Добавляем квест
                        questListItem = ql:addQuestListItem()
                        questListItem:setText("Defeat the first wave")
                    end
                }))
            end)
        end,
        done = function()
            log("Stage 1 done")

            questListItem:setCompleted(true)
            tut:delay(1, function()
                ql:removeQuestListItem(questListItem)

                tut:startNextStage()
            end)
        end
    },
    {-- Выделить телепорт
        check = function()
            return systems.MapSystem:getSelectedGate() ~= nil and systems.MapSystem:getSelectedGate():getType() == enums.GateType.TELEPORT
        end,
        start = function()
            log("Stage 2 start")

            systems.MapSystem:removeHighlights()
            local map = systems.MapSystem:getMap()
            for y = 0, map.heightTiles do
                for x = 0, map.widthTiles do
                    for i = 1, 2 do
                        local gate = map:getGate(x, y, GateSide.values[i])
                        if gate ~= nil and gate:getType() == enums.GateType.TELEPORT then
                            systems.MapSystem:highlightGate(gate)
                        end
                    end
                end
            end

            sm:queue({
                tut.ADVINAS_TAG .. " Enemies can pass through [#00BCD4]Teleports[], each teleport is associated with other teleports of the same color.",
                tut.ADVINAS_TAG .. " [#00BCD4]Select any Teleport[] to view information about it."
            })
            sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                run = function()
                    log("Stage 2 sm continue")
                    questListItem = ql:addQuestListItem()
                    questListItem:setText("Select any Teleport")
                end
            }))
        end,
        done = function()
            log("Stage 2 done")

            systems.GameStateSystem:addCompletedQuest("Q:0.4:1")

            systems.MapSystem:removeHighlights()

            questListItem:setCompleted(true)
            tut:delay(2, function()
                ql:removeQuestListItem(questListItem)

                tut:startNextStage()
            end)
        end
    },
    {-- Победить вторую волну
        check = function()
            return systems.WaveSystem:getCompletedWavesCount() >= 2
        end,
        start = function()
            log("Stage 3 start")

            -- Показываем кнопку вызова волны
            systems.GraphicsSystem.mainUi:showNextWaveButton(false, nil)
            systems.WaveSystem:freezeTimeToNextWave(false)

            -- При вызове волны скрываем кнопку и останавливаем таймер волны
            tut:condition(function() return systems.WaveSystem.wave.waveNumber == 2 end, function()
                systems.GraphicsSystem.mainUi:hideNextWaveButton()
                systems.WaveSystem:freezeTimeToNextWave(true)
            end)

            sm:queue({
                tut.ADVINAS_TAG .. " Now [#00BCD4]deal with the second wave of Fast enemies[]."
            })
            sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                run = function()
                    log("Stage 3 sm continue")

                    -- Добавляем квест
                    questListItem = ql:addQuestListItem()
                    questListItem:setText("Defeat one more wave")
                end
            }))
        end,
        done = function()
            log("Stage 3 done")

            questListItem:setCompleted(true)
            tut:delay(1, function()
                ql:removeQuestListItem(questListItem)
                tut:startNextStage()
            end)
        end
    },
    {-- Выделить барьер
        check = function()
            return systems.MapSystem:getSelectedGate() ~= nil and systems.MapSystem:getSelectedGate():getType() == enums.GateType.BARRIER_TYPE
        end,
        start = function()
            log("Stage 4 start")

            systems.MapSystem:removeHighlights()
            local map = systems.MapSystem:getMap()
            for y = 0, map.heightTiles do
                for x = 0, map.widthTiles do
                    for i = 1, 2 do
                        local gate = map:getGate(x, y, GateSide.values[i])
                        if gate ~= nil and gate:getType() == enums.GateType.BARRIER_TYPE then
                            systems.MapSystem:highlightGate(gate)
                        end
                    end
                end
            end

            sm:queue({
                tut.ADVINAS_TAG .. " As you could see, Fast enemies chose a longer path to bypass [#00BCD4]Barriers[].",
                tut.ADVINAS_TAG .. " [#00BCD4]Barriers can block certain types of enemies[]. Tap on barrier to see more details about it."
            })
            sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                run = function()
                    log("Stage 4 sm continue")
                    questListItem = ql:addQuestListItem()
                    questListItem:setText("Select any Barrier")
                end
            }))
        end,
        done = function()
            log("Stage 4 done")

            systems.MapSystem:removeHighlights()

            questListItem:setCompleted(true)

            tut:delay(2, function()
                ql:removeQuestListItem(questListItem)

                -- Добавляем STRONG в список заблокированных врагов
                local map = systems.MapSystem:getMap()
                for y = 0, map.heightTiles do
                    for x = 0, map.widthTiles do
                        for i = 1, 2 do
                            local gate = map:getGate(x, y, GateSide.values[i])
                            if gate ~= nil and gate:getType() == enums.GateType.BARRIER_TYPE then
                                gate.blockedEnemies:add(enums.EnemyType.STRONG)
                                systems.MapSystem:setGate(gate:getX(), gate:getY(), gate:getSide(), gate)
                            end
                        end
                    end
                end

                sm:queue({
                    tut.ADVINAS_TAG .. " Barriers can also block more than one enemy type. I slightly changed the existing barrier so that it also blocks Strong enemies."
                })
                sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                    run = function()
                        log("Stage 4 sm continue")
                        tut:startNextStage()
                    end
                }))
            end)
        end
    },
    {-- Выделить тайл с ресурсом
        check = function()
            return systems.MapSystem:getSelectedTile() ~= nil and systems.MapSystem:getSelectedTile().type == enums.TileType.SOURCE
        end,
        start = function()
            log("Stage 5 start")

            systems.MapSystem:removeHighlights()
            local map = systems.MapSystem:getMap()
            for y = 0, map.heightTiles - 1 do
                for x = 0, map.widthTiles - 1 do
                    local tile = map:getTile(x, y)
                    if tile ~= nil and tile.type == enums.TileType.SOURCE then
                        systems.MapSystem:highlightTile(tile)
                    end
                end
            end

            tut:delay(1, function()
                sm:queue({
                    tut.ADVINAS_TAG .. " Now let's talk about [#00BCD4]Resources[].",
                    tut.ADVINAS_TAG .. " [#00BCD4]Resources[] is a useful material that I can use to [#00BCD4]research and discover new content[] for you.",
                    tut.ADVINAS_TAG .. " [#00BCD4]Resources[] can be obtained from [#00BCD4]Sources[]. [#00BCD4]Select any Source to view information about it[]."
                })
                sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                    run = function()
                        log("Stage 5 sm continue")
                        questListItem = ql:addQuestListItem()
                        questListItem:setText("Select any Source tile")
                    end
                }))
            end)
        end,
        done = function()
            log("Stage 5 done")

--            systems.MapSystem:removeHighlights()

            systems.GameStateSystem:addCompletedQuest("Q:0.4:2")

            questListItem:setCompleted(true)
            tut:delay(2, function()
                ql:removeQuestListItem(questListItem)

                tut:startNextStage()
            end)
        end
    },
    {-- Построить майнер
        check = function()
            return systems.MinerSystem.miners.size ~= 0
        end,
        update = function()
            local sideMenuOffscreen = systems.GraphicsSystem.sideMenu:isOffscreen()
            if s6_sideMenuWasOffscreen ~= sideMenuOffscreen then
                -- Меняем подсветку элементов UI
                managers.UiManager:removeAllHighlights()

                if sideMenuOffscreen then
                    -- Меню скрыто - подсвечиваем кнопку открытия меню
                    managers.UiManager:addHighlight(managers.UiManager:findActor("side_menu_toggle_button"))
                else
                    -- Меню открыто, подсвечиваем кнопку постройки
                    managers.UiManager:addHighlight(managers.UiManager:findActor("build_menu_miner_build_button_SCALAR"))
                end
                s6_sideMenuWasOffscreen = sideMenuOffscreen
            end
        end,
        start = function()
            log("Stage 6 start")

            tut:delay(1, function()
                managers.UiManager:findActor("build_menu_miner_build_buttons"):setVisible(true)

                -- Подсвечиваем кнопку постройки майнера
                s6_sideMenuWasOffscreen = not systems.GraphicsSystem.sideMenu:isOffscreen()

                sm:queue({
                    tut.ADVINAS_TAG .. " You'll use [#00BCD4]Miners[] in order to dig out [#00BCD4]Resources from Sources[].",
                    tut.ADVINAS_TAG .. " We have enough coins to build our first Miner. [#00BCD4]Select any Source and build a Miner on it.[]"
                })
                sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                    run = function()
                        log("Stage 6 sm continue")
                        questListItem = ql:addQuestListItem()
                        questListItem:setText("Build a Miner")
                    end
                }))
            end)
        end,
        done = function()
            log("Stage 6 done")

            systems.GameStateSystem:addCompletedQuest("Q:0.4:3")

            systems.MapSystem:removeHighlights()
            managers.UiManager:removeAllHighlights()

            questListItem:setCompleted(true)
            tut:delay(2, function()
                ql:removeQuestListItem(questListItem)

                sm:queue({
                    tut.ADVINAS_TAG .. " You have a limited amount of [#00BCD4]Miners[] of each type, [#00BCD4]Researches[] can increase their number."
                })
                sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                    run = function()
                        tut:startNextStage()
                    end
                }))
            end)
        end
    },
    {-- Дождаться установки майнера
        check = function()
            return systems.MinerSystem.miners.size ~= 0 and systems.MinerSystem.miners:get(0):isPrepared()
        end,
        start = function()
            log("Stage 7 start")

            tut:delay(1, function()
                systems.GraphicsSystem.mainUi:showNextWaveButton(false, nil)
                systems.WaveSystem:freezeTimeToNextWave(false)
                systems.GraphicsSystem.mainUi:showGameSpeedButton(false, nil)

                sm:queue({
                    tut.ADVINAS_TAG .. " [#00BCD4]Miners[] take some time to install before they can start digging.",
                    tut.ADVINAS_TAG .. " Installation progress is displayed as a progress bar on top of [#00BCD4]Miner menu[], let's wait for it to install."
                })
                sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                    run = function()
                        log("Stage 7 sm continue")
                        questListItem = ql:addQuestListItem()
                        questListItem:setText("Wait for Miner to install")
                    end
                }))
            end)
        end,
        done = function()
            log("Stage 7 done")

            questListItem:setCompleted(true)
            tut:delay(1, function()
                ql:removeQuestListItem(questListItem)

                sm:queue({
                    tut.ADVINAS_TAG .. " Each next [#00BCD4]Miner[] you build will cost more coins. You can also upgrade [#00BCD4]Miners[] to increase their speed.",
                    tut.ADVINAS_TAG .. " Now just watch my tower deals with the remaining enemies and your Miner digs some Resources for us."
                })
                sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                    run = function()
                        log("Stage 7 sm continue")
                        questListItem = ql:addQuestListItem()
                        questListItem:setText("Defeat remaining enemies")
                    end
                }))
                tut:startNextStage()
            end)
        end
    }
})

local stateSystemListener = luajava.createProxy(GNS .. "systems.GameStateSystem$GameStateSystemListener", {
    realUpdate = function(deltaTime)
        tut:update(deltaTime)
    end
})

setupListener = function()
    sm = systems.GraphicsSystem.storylineMessages
    ql = systems.GraphicsSystem.questList

    -- Приводим все в начальное состояние
    systems.GraphicsSystem.mainUi:showMessagesButton(false, nil)
    systems.GraphicsSystem.mainUi:hideGameSpeedButton()
    managers.UiManager:findActor("build_menu_miner_build_buttons"):setVisible(false)

    tut:start()

    -- Слушаем изменения
    systems.GameStateSystem:addListener(stateSystemListener)

    --
    addEventHandler("SystemDispose", disposeListener)
    log(TAG .. "Started")
end

disposeListener = function()
    systems.GameStateSystem:removeListener(stateSystemListener)
    --
    removeEventHandler("SystemPostSetup", setupListener)
    removeEventHandler("SystemDispose", disposeListener)
    log(TAG .. "Stopped")
end

addEventHandler("SystemPostSetup", setupListener)