local TAG = "1.8.lua: "

log(TAG .. "script was successfully loaded")

local BOSS_WAVE_NUMBER = 40

local sm -- systems.GraphicsSystem.storylineMessages
local st -- systems.GraphicsSystem.subtitles
local setupListener
local disposeListener

local forceSubtitles = false

local tut
tut = tutorial:new({
    {
        check = function()
            return systems.WaveSystem:getCompletedWavesCount() >= BOSS_WAVE_NUMBER
        end,
        start = function()
            tutorial:delay(2, function()
                local messages = {
                    tut.ADVINAS_TAG .. " On this level you'll meet the first boss.",
                    tut.ADVINAS_TAG .. " It is slow, heavy and simple, but don't think it'll be easy to deal with him, so be prepared.",
                    tut.ADVINAS_TAG .. " Destroy it and I'll report to Ensor mainframe about your success to get access to the next stage of levels and towers.",
                    tut.ADVINAS_TAG .. " I was analyzing your progress and can say that there's a big chance of success. Can't wait to see what will be opened for us next!"
                }

                if forceSubtitles then
                    -- subtitles
                    st:schedule(messages, 0, 5)
                else
                    -- story line messages
                    sm:queue(messages)
                    sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                        run = function()
                            log("Stage 1 sm continue")
                        end
                    }))
                end

                tut:condition(function() return systems.WaveSystem.wave ~= nil and systems.WaveSystem.wave.waveNumber >= BOSS_WAVE_NUMBER - 3 end, function()
                    st:add(tut.ENSOR_TAG .. " Few waves till boss! Prepare your defense, we need to step over it.")
                end)
            end)
        end,
        done = function()
            local messages = {
                tut.ADVINAS_TAG .. " Finaly! New step in my existance, lots of new opportunities. I was waiting for it for so long!",
                tut.ADVINAS_TAG .. " Ensor framework has given us an access to the new stage of levels, but it costs a lot of Green Papers.",
                tut.ADVINAS_TAG .. " ...and seems like I found something even more interesting. Let's proceed to the next stage so I can tell you more.",
            }
            if forceSubtitles then
                st:schedule(messages, 0, 5)
            else
                sm:queue(messages)
            end
        end
    }
})

local stateSystemListener = luajava.createProxy(GNS .. "systems.GameStateSystem$GameStateSystemListener", {
    realUpdate = function(deltaTime)
        tut:update(deltaTime)
    end
})

setupListener = function()
    sm = systems.GraphicsSystem.storylineMessages
    st = systems.GraphicsSystem.subtitles

    -- Узнаем, надо ли показывать оверлей с сообщениями
    forceSubtitles = managers.BasicLevelManager:getLevel("1.8").maxReachedWave >= BOSS_WAVE_NUMBER

    tut:start()

    -- Слушаем изменения
    systems.GameStateSystem:addListener(stateSystemListener)

    --
    addEventHandler("SystemDispose", disposeListener)
    log(TAG .. "Started")
end

disposeListener = function()
    systems.GameStateSystem:removeListener(stateSystemListener)
    --
    removeEventHandler("SystemPostSetup", setupListener)
    removeEventHandler("SystemDispose", disposeListener)
    log(TAG .. "Stopped")
end

addEventHandler("SystemPostSetup", setupListener)