local TAG = "2.8.lua: "

log(TAG .. "script was successfully loaded")

local BOSS_WAVE_NUMBER = 46

local sm -- systems.GraphicsSystem.storylineMessages
local st -- systems.GraphicsSystem.subtitles
local setupListener
local disposeListener

local forceSubtitles = false

local tut
tut = tutorial:new({
    {
        check = function()
            return systems.WaveSystem:getCompletedWavesCount() >= BOSS_WAVE_NUMBER
        end,
        start = function()
            local messages = {
                tut.ADVINAS_TAG .. " New boss incoming!",
                tut.ADVINAS_TAG .. " It is a representation of a blockchain, actually a chain of enemies.",
                tut.ADVINAS_TAG .. " Data we need is stored in one first block, other blocks add layers of encryption and make it harder to crack the main block.",
                tut.ADVINAS_TAG .. " Just try to drop those encryption layers before going for the main block. Good luck!",
            }
            if forceSubtitles then
                st:schedule(messages, 2, 5)
            else
                tutorial:delay(2, function()
                    sm:queue(messages)
                    sm:runOnContinue(luajava.createProxy("java.lang.Runnable", {
                        run = function()
                            log("Stage 1 sm continue")
                        end
                    }))
                end)
            end

            tut:condition(function() return systems.WaveSystem.wave ~= nil and systems.WaveSystem.wave.waveNumber >= BOSS_WAVE_NUMBER - 3 end, function()
                st:add(tut.ENSOR_TAG .. " You have few waves to prepare before boss comes. Remember - main block is protected by smaller encryption layers!")
            end)
        end,
        done = function()
            local messages = {
                tut.ADVINAS_TAG .. " Great job!",
                tut.ADVINAS_TAG .. " We got access to the next stage of levels, also I'm now able to handle new type of resource - Matrix.",
                tut.ADVINAS_TAG .. " Let's move on for the new skies!",
            }
            if forceSubtitles then
                st:schedule(messages, 0, 5)
            else
                sm:queue(messages)
            end
        end
    }
})

local stateSystemListener = luajava.createProxy(GNS .. "systems.GameStateSystem$GameStateSystemListener", {
    realUpdate = function(deltaTime)
        tut:update(deltaTime)
    end
})

setupListener = function()
    sm = systems.GraphicsSystem.storylineMessages
    st = systems.GraphicsSystem.subtitles

    -- Узнаем, надо ли показывать оверлей с сообщениями
    forceSubtitles = managers.BasicLevelManager:getLevel("2.8").maxReachedWave >= BOSS_WAVE_NUMBER

    tut:start()

    -- Слушаем изменения
    systems.GameStateSystem:addListener(stateSystemListener)

    --
    addEventHandler("SystemDispose", disposeListener)
    log(TAG .. "Started")
end

disposeListener = function()
    systems.GameStateSystem:removeListener(stateSystemListener)
    --
    removeEventHandler("SystemPostSetup", setupListener)
    removeEventHandler("SystemDispose", disposeListener)
    log(TAG .. "Stopped")
end

addEventHandler("SystemPostSetup", setupListener)