local TAG = "balancing: "

local MainUiLayer = luajava.bindClass(GNS .. "managers.UiManager$MainUiLayer")
local Color = luajava.bindClass("com.badlogic.gdx.graphics.Color")
local Label = luajava.bindClass("com.badlogic.gdx.scenes.scene2d.ui.Label")
local AimStrategy = luajava.bindClass(GNS .. "Tower$AimStrategy")
local Gdx = luajava.bindClass("com.badlogic.gdx.Gdx")
local FileType = luajava.bindClass("com.badlogic.gdx.Files$FileType")
local GameOverReason = luajava.bindClass(GNS .. "systems.GameStateSystem$GameOverReason")
local WaveSystemMode = luajava.bindClass(GNS .. "systems.WaveSystem$Mode")
local JavaArray = luajava.bindClass("java.lang.reflect.Array")
local PredefinedWaveTemplate = luajava.bindClass(GNS .. "WaveTemplates$PredefinedWaveTemplate")

log(TAG .. "script was successfully loaded")

local setupListener
local disposeListener

local oneWaveMode = true
local fastForwardEnabled = true
local enemiesCountMultiplier = 1

local isInit = false

local towerTypes = {
    enums.TowerType.BASIC, enums.TowerType.SNIPER, enums.TowerType.CANNON, enums.TowerType.VENOM, enums.TowerType.SPLASH, enums.TowerType.BLAST,
    enums.TowerType.MULTISHOT, enums.TowerType.MINIGUN, enums.TowerType.AIR, enums.TowerType.TESLA, enums.TowerType.MISSILE, enums.TowerType.FLAMETHROWER,
    enums.TowerType.LASER
}
local enemyTypes
if oneWaveMode then
    enemyTypes = {
       "RegularLow",
        "RegularMedium",
        "RegularHigh"
    }
else
    enemyTypes = {
        "RegularLow",
        "RegularMedium",
        "RegularHigh"
        --    "StrongOnly",
        --    "AirOnly"
    }
end

local upgradeLevels = {
    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10
}
local experienceLevels = {
    1
--    1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20
}
local abilityIndices = {
    -1
--    -1, 0, 1, 2
}

-- [тип башни][тип врага][уровень улучшения башни][уровень опыта башни][установленный навык] = [потраченные монеты, достигнутая волна, нанесенный урон]
local testResults = {}
local current = {
    towerType = 1,
    enemyType = 1,
    upgradeLevel = 1,
    experienceLevel = 1,
    abilityIndex = 1
}

-- Создаем UI
local overlayUiLayer = managers.UiManager:addLayer(MainUiLayer.OVERLAY, 160, "Balancing overlay");
overlayUiLayer:getTable():setBackground(managers.AssetManager:getDrawable("blank"):tint(luajava.newInstance("com.badlogic.gdx.graphics.Color", 0.1, 0.1, 0.1, 0.5)))

local progressLabel = luajava.newInstance("com.badlogic.gdx.scenes.scene2d.ui.Label", "", managers.AssetManager:getLabelStyle(CFG.FONT_SIZE_MEDIUM));
progressLabel:setAlignment(1)
overlayUiLayer:getTable():add(progressLabel)
--overlayUiLayer:getTable():setDebug(true, true)

local updateStatusLabel

-- Сохранить текущие результаты и начать новый этап (перезагрузить экран), если есть
local startNextBenchmarkStage = function()
    -- Сохраняем результаты
    local towerTypeName = towerTypes[current.towerType]:name()
    local enemyTypeName = enemyTypes[current.enemyType]
    local upgradeLevelName = tostring(upgradeLevels[current.upgradeLevel])
    local experienceLevelName = tostring(experienceLevels[current.experienceLevel])
    local abilityIndexName = tostring(abilityIndices[current.abilityIndex])

    if testResults[towerTypeName] == nil then
        testResults[towerTypeName] = {}
    end
    if testResults[towerTypeName][enemyTypeName] == nil then
        testResults[towerTypeName][enemyTypeName] = {}
    end
    if testResults[towerTypeName][enemyTypeName][upgradeLevelName] == nil then
        testResults[towerTypeName][enemyTypeName][upgradeLevelName] = {}
    end
    if testResults[towerTypeName][enemyTypeName][upgradeLevelName][experienceLevelName] == nil then
        testResults[towerTypeName][enemyTypeName][upgradeLevelName][experienceLevelName] = {}
    end

    testResults[towerTypeName][enemyTypeName][upgradeLevelName][experienceLevelName][abilityIndexName] = {
        systems.StatisticsSystem:getCurrentGameStatistic(enums.StatisticsType.TMS),
        systems.WaveSystem:getCompletedWavesCount(),
        systems.StatisticsSystem:getCurrentGameStatistic(enums.StatisticsType.TDD)
    }

    -- Увеличиваем индексы
    current.abilityIndex = current.abilityIndex + 1
    if current.abilityIndex == #abilityIndices + 1 then
        current.abilityIndex = 1
        current.experienceLevel = current.experienceLevel + 1
        if current.experienceLevel == #experienceLevels + 1 then
            current.experienceLevel = 1
            current.upgradeLevel = current.upgradeLevel + 1
            if current.upgradeLevel == #upgradeLevels + 1 then
                current.upgradeLevel = 1
                current.enemyType = current.enemyType + 1
                if current.enemyType == #enemyTypes + 1 then
                    current.enemyType = 1
                    current.towerType = current.towerType + 1
                end
            end
        end
    end

    if current.towerType == #towerTypes + 1 then
        -- Конец тестирования, выгружаем в json
--        log(utils.printr(testResults, 1))

        local strParts = {}
        table.insert(strParts, "{\n")
        for k, towerType in pairs(towerTypes) do
            local towerTypeRes = testResults[towerType:name()]

            table.insert(strParts, " \"")
            table.insert(strParts, towerType:name())
            table.insert(strParts, "\":{\n")

            for kk, waveType in pairs(enemyTypes) do
                local waveTypeRes = towerTypeRes[waveType]

                table.insert(strParts, "  \"")
                table.insert(strParts, waveType)
                table.insert(strParts, "\":{\n")

                for kkk, upgradeLevel in pairs(upgradeLevels) do
                    local upgradeLevelRes = waveTypeRes[tostring(upgradeLevel)]

                    table.insert(strParts, "   \"")
                    table.insert(strParts, tostring(upgradeLevel))
                    table.insert(strParts, "\":{\n")

                    for kkkk, experienceLevel in pairs(experienceLevels) do
                        local experienceLevelRes = upgradeLevelRes[tostring(experienceLevel)]

                        table.insert(strParts, "    \"")
                        table.insert(strParts, tostring(experienceLevel))
                        table.insert(strParts, "\":{\n")

                        for kkkkk, abilityIndex in pairs(abilityIndices) do
                            local res = experienceLevelRes[tostring(abilityIndex)]

                            table.insert(strParts, "     \"")
                            table.insert(strParts, tostring(abilityIndex))
                            table.insert(strParts, "\":[")

                            table.insert(strParts, tostring(res[1]))
                            table.insert(strParts, ",")
                            table.insert(strParts, tostring(res[2]))
                            table.insert(strParts, ",")
                            table.insert(strParts, tostring(res[3]))
                            table.insert(strParts, "")

                            table.insert(strParts, "]")
                            if kkkkk ~= #abilityIndices then
                                table.insert(strParts, ",")
                            end
                            table.insert(strParts, "\n")
                        end

                        table.insert(strParts, "    }")
                        if kkkk ~= #experienceLevels then
                            table.insert(strParts, ",")
                        end
                        table.insert(strParts, "\n")
                    end

                    table.insert(strParts, "   }")
                    if kkk ~= #upgradeLevels then
                        table.insert(strParts, ",")
                    end
                    table.insert(strParts, "\n")
                end

                table.insert(strParts, "  }")
                if kk ~= #enemyTypes then
                    table.insert(strParts, ",")
                end
                table.insert(strParts, "\n")
            end

            table.insert(strParts, " }")
            if k ~= #towerTypes then
                table.insert(strParts, ",")
            end
            table.insert(strParts, "\n")
        end

        table.insert(strParts, "}")
        local jsonStr = table.concat(strParts)

        -- Записываем в файл
        Gdx.files:getFileHandle("../../asset sources/benchmark/results.json", FileType.Local):writeString(jsonStr)
        Gdx.files:getFileHandle("../../asset sources/benchmark/results.js", FileType.Local):writeString("BenchmarkResults = " .. jsonStr .. ";")

        -- Останавливаем скрипт
        removeEventHandler("SystemPostSetup", setupListener)
        managers.UiManager:removeLayer(overlayUiLayer)
        log(TAG .. "script stopped")
    else
        -- Следующий этап
        systems.GameStateSystem:restartGame(true)
    end
end

updateStatusLabel = function()
    local totalStages = #abilityIndices * #experienceLevels * #upgradeLevels * #enemyTypes * #towerTypes
    local currentStage = current.abilityIndex
            + (current.experienceLevel - 1) * #abilityIndices
            + (current.upgradeLevel - 1) * (#abilityIndices * #experienceLevels)
            + (current.enemyType - 1) * (#abilityIndices * #experienceLevels * #upgradeLevels)
            + (current.towerType - 1) * (#abilityIndices * #experienceLevels * #upgradeLevels * #enemyTypes)

    local cw = 0
    if systems ~= nil then cw = systems.WaveSystem:getCompletedWavesCount() end

    if current.towerType <= #towerTypes then
        progressLabel:setText(
            "Wave: " .. cw .. "\n"
                    .. towerTypes[current.towerType]:name() .. " - " .. enemyTypes[current.enemyType] .. "\n"
                    .. currentStage .. " / " .. totalStages
        )
    end

    if cw == 300 then
        systems.GameStateSystem:triggerGameOver(GameOverReason.MANUAL)
    elseif cw > 200 then
        progressLabel:setColor(Color.RED)
    else
        progressLabel:setColor(Color.WHITE)
    end
end

local stateSystemListener = luajava.createProxy(GNS .. "systems.GameStateSystem$GameStateSystemListener", {
    gameOver = function()
        startNextBenchmarkStage()
    end
})

local waveSystemListener = luajava.createProxy(GNS .. "systems.WaveSystem$WaveSystemListener", {
    waveCompleted = function()
        updateStatusLabel()
    end
})

setupListener = function()
    -- Игра началась
    if not isInit then
        -- Первый запуск
        managers.TowerManager:reloadTowerStats()
    end

    -- Слушаем изменения
    log(TAG .. "Game screen started")
    systems.GameStateSystem:addListener(stateSystemListener)
    systems.WaveSystem:addListener(waveSystemListener)

    -- Устанавливаем типы врагов
    if oneWaveMode then
        systems.WaveSystem.mode = WaveSystemMode.PREDEFINED
        local templatesArr = JavaArray:newInstance(PredefinedWaveTemplate, 1)

        local enemyHealth = 3000
--        if upgradeLevels[current.upgradeLevel] > 0 then
            enemyHealth = enemyHealth + upgradeLevels[current.upgradeLevel] * upgradeLevels[current.upgradeLevel] * 1500
--        end

        if #experienceLevels > 1 then
            enemyHealth = enemyHealth * 5
        end

        local enemyGroup1
        local enemyGroup2
        local enemyGroup3
        if enemyTypes[current.enemyType] == "RegularLow" then
            enemyGroup1 = luajava.newInstance(GNS .. "EnemyGroup", enums.EnemyType.REGULAR, 1.0, enemyHealth * 2 / 24, math.floor(50 * enemiesCountMultiplier), 0,        1.4 * 3, 1, 1, 1)
            enemyGroup2 = luajava.newInstance(GNS .. "EnemyGroup", enums.EnemyType.REGULAR, 1.0, enemyHealth * 2 / 8,  math.floor(50 * enemiesCountMultiplier), 1.4,      1.4 * 3, 1, 1, 1)
            enemyGroup3 = luajava.newInstance(GNS .. "EnemyGroup", enums.EnemyType.REGULAR, 1.0, enemyHealth * 2,      math.floor(50 * enemiesCountMultiplier), 1.4 * 2,  1.4 * 3, 1, 1, 1)
        elseif enemyTypes[current.enemyType] == "RegularMedium" then
            enemyGroup1 = luajava.newInstance(GNS .. "EnemyGroup", enums.EnemyType.REGULAR, 1.0, enemyHealth / 24,     math.floor(100 * enemiesCountMultiplier), 0,       0.7 * 3, 1, 1, 1)
            enemyGroup2 = luajava.newInstance(GNS .. "EnemyGroup", enums.EnemyType.REGULAR, 1.0, enemyHealth / 8,      math.floor(100 * enemiesCountMultiplier), 0.7,     0.7 * 3, 1, 1, 1)
            enemyGroup3 = luajava.newInstance(GNS .. "EnemyGroup", enums.EnemyType.REGULAR, 1.0, enemyHealth,          math.floor(100 * enemiesCountMultiplier), 0.7 * 2, 0.7 * 3, 1, 1, 1)
        elseif enemyTypes[current.enemyType] == "RegularHigh" then
            enemyGroup1 = luajava.newInstance(GNS .. "EnemyGroup", enums.EnemyType.REGULAR, 1.0, enemyHealth / 2 / 24, math.floor(200 * enemiesCountMultiplier), 0,        0.35 * 3, 1, 1, 1)
            enemyGroup2 = luajava.newInstance(GNS .. "EnemyGroup", enums.EnemyType.REGULAR, 1.0, enemyHealth / 2 / 8,  math.floor(200 * enemiesCountMultiplier), 0.35,     0.35 * 3, 1, 1, 1)
            enemyGroup3 = luajava.newInstance(GNS .. "EnemyGroup", enums.EnemyType.REGULAR, 1.0, enemyHealth / 2,      math.floor(200 * enemiesCountMultiplier), 0.35 * 2, 0.35 * 3, 1, 1, 1)
        end

        templatesArr[1] = luajava.newInstance(GNS .. "WaveTemplates$PredefinedWaveTemplate", enemyGroup1, enemyGroup2, enemyGroup3)
        systems.WaveSystem.predefinedWaveTemplates = templatesArr
        systems.WaveSystem:resetNextWavesCache()
        systems.GameStateSystem:addHealth(50000)
    else
        systems.WaveSystem:setForcedTemplate(enemyTypes[current.enemyType])
    end

    -- Устанавливаем башни
    local map = systems.MapSystem:getMap()
    for y = 0, systems.MapSystem:getMap().heightTiles - 1 do
        for x = 0, systems.MapSystem:getMap().widthTiles - 1 do
            local tile = map:getTile(x, y)
            if tile ~= nil and tile.type == enums.TileType.PLATFORM and tile.building == nil then
                local tower = systems.TowerSystem:buildTower(towerTypes[current.towerType], AimStrategy.RANDOM, x, y)

                tower:addExperienceToMatchLevel(experienceLevels[current.experienceLevel])
                for i = 1, upgradeLevels[current.upgradeLevel] do
                    systems.TowerSystem:upgradeTower(tower)
                end
                if abilityIndices[current.abilityIndex] ~= -1 then
                    systems.TowerSystem:setAbilityInstalled(tower, abilityIndices[current.abilityIndex], true)
                end
            end
        end
    end

    -- Начинаем волны автоматом
    systems.WaveSystem:setAutoForceWaveEnabled(true)

    -- Ставим максимальную скорость
--    systems.GameStateSystem:setGameSpeed(512)

    -- Включаем быструю перемотку
    if fastForwardEnabled then
        systems.GameStateSystem:startFastForwarding(500000)
    end
    -- Показываем процесс
    updateStatusLabel()
--    managers.UiManager.loadingOverlay:show(currentStage / totalStages, )

    if isInit then return end

    -- Первый раз за все игры запустилось
    log(TAG .. "Started once")

    addEventHandler("SystemDispose", disposeListener)
    isInit = true
end

disposeListener = function()
    systems.GameStateSystem:removeListener(stateSystemListener)
    systems.WaveSystem:removeListener(waveSystemListener)

    removeEventHandler("SystemDispose", disposeListener)
    log(TAG .. "Stopped")
end

addEventHandler("SystemPostSetup", setupListener)