GNS = "com.prineside.tdi2." -- Game name space
CFG = luajava.bindClass(GNS .. "Config")
i18n = managers.LocaleManager.i18n

-- com.prineside.tdi2.enums.*
local enumNames = { 
    "AbilityType",
    "AchievementType",
    "ActionType",
    "BuffType",
    "BuildingType",
    "CaseType",
    "DamageType",
    "EnemyType",
    "ExplosionType",
    "GameValueType",
    "GateType",
    "GeneralizedTowerStatType",
    "ItemType",
    "MinerType",
    "ProjectileType",
    "RequirementType",
    "ResearchCategoryType",
    "ResearchType",
    "ResourceType",
    "ShapeType",
    "SoundType",
    "SpaceTileBonusType",
    "StatisticsType",
    "TileType",
    "TowerStatType",
    "TowerType",
    "TrophyType",
    "UnitType"
};
_G.enums = {}
for _, v in pairs(enumNames) do
    _G.enums[v] = luajava.bindClass(GNS .. "enums." .. v)
end

-- _G is a global namespace table, its fields can also be accessed without _G prefix ("_G.utils" == "utils")
_G.utils = {
    --[[
      Print "var" recursively if it is a table or as usual string if not
      tabSize - number of spaces for pretty print (may be nil)
      visitedTables - visited tables as keys to not print them again (may be nil)
    --]]
    printr = function(var, tabSize, visitedTables)
        if tabSize == nil then tabSize = 0 end
        if visitedTables == nil then visitedTables = {} end

        if type(var) == "table" then
            if visitedTables[var] ~= nil then
                return "..."
            end
            visitedTables[var] = true

            local currentTab = string.rep("  ", tabSize)
            local nextTab = string.rep("  ", tabSize + 1)
            local out = {}
            local cnt = 0
            for k, v in pairs(var) do
                local lineParts = {}
                table.insert(lineParts, nextTab)
                table.insert(lineParts, tostring(k))
                table.insert(lineParts, " = ")
                table.insert(lineParts, _G.utils.printr(v, tabSize + 1, visitedTables))
                table.insert(out, table.concat(lineParts, ""))
                cnt = cnt + 1
            end

            if cnt == 0 then
                return currentTab .. tostring(var) .. " (size: 0) {}"
            else
                return currentTab .. tostring(var) .. " (size: " .. cnt .. ") {\n" .. table.concat(out, "\n") .. "\n" .. currentTab .. "}"
            end
        elseif type(var) == "userdata" then
            local descr = tostring(var)
            local fields = reflection.getFields(var)
            if fields.length ~= 0 then
                descr = descr .. "\n  Fields:\n"
                for i = 1, fields.length do
                    descr = descr .. "    " .. fields[i]:getName() .. "\n"
                end
            end

            local methods = reflection.getMethods(var)
            if methods.length ~= 0 then
                descr = descr .. "\n  Methods:\n"
                for i = 1, methods.length do
                    descr = descr .. "    " .. methods[i]:getName() .. "("
                    local paramTypes = methods[i]:getParameterTypes()
                    for j = 1, paramTypes.length do
                        if j ~= 1 then descr = descr .. ", " end

                        descr = descr .. tostring(paramTypes[j])
                    end
                    descr = descr .. ")\n"
                end
            end

            return descr
        else
            return tostring(var)
        end
    end,

    stringsplit = function (inputstr, sep)
        if sep == nil then
            sep = "%s"
        end
        local t={} ; i=1
        for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
            t[i] = str
            i = i + 1
        end
        return t
    end,

    -- Убрать все nil из таблицы и сдвинуть значения
    compactnil = function (t)
        local j = 0
        local n = #t
        for i = 1,n do
            if t[i] ~= nil then
                j = j+1
                t[j] = t[i]
            end
        end
        for i = j + 1, n do
            t[i] = nil
        end
    end
}

_G.tutorial = {
    ADVINAS_TAG = "[#ffc107]ADVINAS:[]",
    ENSOR_TAG = "[#03A9F4]Ensor Inc.:[]",

    new = function(self, tutorialStages)
        tutorialStages = tutorialStages or {}

        local o = {}
        setmetatable(o, self)
        self.__index = self

        self.stages = tutorialStages;
        self.currentStage = 1;
        self.updateTimeAccumulator = 0;
        self.currentStageIsDone = false;
        self.delayedFunctions = {};
        self.conditions = {};

        return o
    end,

    start = function(self)
        for i = 1, #self.stages do
            self.currentStage = i
            if not self.stages[i].check() then
                self.stages[i].start()
                break
            end
        end
    end,

    delay = function(self, delayInSeconds, cb)
        table.insert(self.delayedFunctions, {
            delay = delayInSeconds,
            cb = cb
        })
    end,

    condition = function(self, condition, cb)
        self.conditions[#self.conditions + 1] = {
            condition = condition,
            cb = cb
        }
    end,

    startNextStage = function(self)
        self.currentStage = self.currentStage + 1
        if self.stages[self.currentStage] ~= nil then
            self.currentStageIsDone = false
            self.stages[self.currentStage].start()
        else
            log("tutorial - no stages left")
        end
    end,

    update = function(self, realDeltaTime)
        for i = 1, #self.delayedFunctions do
            local e = self.delayedFunctions[i]
            if e ~= nil then
                e.delay = e.delay - realDeltaTime
                if e.delay <= 0 then
                    self.delayedFunctions[i] = nil
                    e.cb()
                end
            end
        end

        utils.compactnil(self.delayedFunctions)

        for i = 1, #self.conditions do
            local e = self.conditions[i]
            if e ~= nil then
                if e.condition() then
                    e.cb()
                    self.conditions[i] = nil
                end
            end
        end
        utils.compactnil(self.conditions)

        self.updateTimeAccumulator = self.updateTimeAccumulator + realDeltaTime
        if self.updateTimeAccumulator > 0.5 then
            self.updateTimeAccumulator = 0

            if self.stages[self.currentStage] ~= nil and not self.currentStageIsDone then
                -- Еще есть задания
                if self.stages[self.currentStage].check() then
                    self.stages[self.currentStage].done()
                    self.currentStageIsDone = true
                else
                    local updateHandler = self.stages[self.currentStage].update
                    if updateHandler ~= nil then updateHandler(realDeltaTime) end
                end
            end
        end
    end
}